/*
	(C) 2004 - 2008 Thomas Bleeker
	MadWizard.org
	
	Released under the BSD license with the exception of parts of the code 
	not written by Thomas Bleeker (www.madwizard.org), which are marked in
	the source code. See license.txt for details.
*/

#include <GL/glut.h>
#include <stdio.h>
#include <util/glutil.h>
#include "3dclasses.h"
#include <iostream>
#include "CGWorld.h"
#include "Camera.h"
#include "InputManager.h"
#include <string>
#include "Statistics.h"
#include "Dino.h"


using namespace std;

CGWorld g_world;
InputManager	g_inputManager(&g_world);

CGPath *g_pDinoPath;
Dino *g_pDino;

const Vertex3 CAMERA_POS(0, 40, 0);
const float   CAMERA_YAW = 0.0, CAMERA_PITCH = 0.0;

const char WORLD_FILE[] = "../models/zoo.cgworld";

#ifdef _DEBUG
const char WINDOW_TITLE[] = "The Zoo !!! Warning: running in debug mode seriously harms performance !!! ";
#else
const char WINDOW_TITLE[] = "The Zoo";
#endif


// Draws a string at x,y. String may use \n for newlines	
void outputText(int x, int y, const char *string)
{
	int len, i;
	len = (int) strlen(string);

	int lines = 1;
	for (i = 0; i < len; i++)
	{
		if (string[i]=='\n') 
			lines++;
	}

	int line = 0;
	glRasterPos2f(x, y - line * 16);
	for (i = 0; i < len; i++)
	{
		char c = string[i];
		if (c=='\n')
		{
			line++;
			glRasterPos2f(x, y - line * 16);
		}
		else
		{
			glutBitmapCharacter(GLUT_BITMAP_9_BY_15, c);
		}
	}
}

void showStats()
{
	// Disable some stuff
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	
	// Setup ortho projection
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, glutGet(GLUT_WINDOW_WIDTH), 0, glutGet(GLUT_WINDOW_HEIGHT),  -10, 10);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// Write statistics
	glColor3f(1,0,0);
	string str = Statistics::instance().getFrameString();
	outputText(4,glutGet(GLUT_WINDOW_HEIGHT) - 20, str.c_str());
}


void display(void)
{
	glClearColor(0.0, 0.0, 0.4, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Update stats
	Statistics::instance().onNewFrame(g_world.getTimer().getTime());

	// Update input manager
	g_inputManager.onFrame();

	// Render the world
	g_world.render();
	
	// Render the dino
	g_pDino->draw();

	// Show stats
	if (g_inputManager.shouldShowStats())
		showStats();

	// Finish
	Statistics::instance().resetFrameStats();
	glutSwapBuffers();
}

void animateDinosaur(float time)
{
	if (g_pDinoPath)
	{
		Vertex3 point = g_pDinoPath->getPointAt(time / 30.0f);
		Vector3 dir = g_pDinoPath->getDirectionAt(time / 30.0f);

		g_pDino->setPosition(point);
		g_pDino->setDirection(dir);
		g_pDino->setPhase(time );
	}
}

void animateAquarium(float time)
{
	CGDynamicObject *pObj = NULL;
	CGPath *pPath = NULL;

	// Animate bubbles
	pObj = g_world.getDynamicObject("bubbles");
	if (pObj)
		pObj->setTextureShift(0, -fmod(time/2.0f, 1.0f));
	pObj = g_world.getDynamicObject("bubbles2");
	if (pObj)
		pObj->setTextureShift(0, -fmod(time/2.1f + 0.3f, 1.0f));
	// Animate water
	pObj = g_world.getDynamicObject("water");
	if (pObj)
		pObj->setTextureShift(sin(time/2.0f) / 10.0f, cos(time * 0.65f)/ 10.0f);

	pObj = g_world.getDynamicObject("water2");
	if (pObj)
		pObj->setTextureShift(sin(time/2.5f) / 10.0f, cos(time * 0.50f)/ 10.0f);

	// Animate water lights
	pObj = g_world.getDynamicObject("waterlight1");
	if (pObj)
		pObj->setTextureShift(0, fmod(time/10.0f, 1.0f));

	pObj = g_world.getDynamicObject("waterlight2");
	if (pObj)
		pObj->setTextureShift(0, fmod(time/10.0f, 1.0f));

	// Dirt in the water
	pObj = g_world.getDynamicObject("waterDirt");
	if (pObj)
		pObj->setTextureShift(fmod(time/10.0f, 1.0f),sin(time/4.0f) * 0.5f);
	
	pObj = g_world.getDynamicObject("waterDirt2");
	if (pObj)
		pObj->setTextureShift(fmod(time/11.0f, 1.0f),cos(time/7.0f) * 0.5f);

	// Animate fishes
	pObj = g_world.getDynamicObject("nemo1");
	pPath = g_world.findPath("fish1");
	if (pObj && pPath)
	{
		pObj->setPosition(pPath->getPointAt(time / 25.0f));
		pObj->setDirection(pPath->getDirectionAt(time / 25.0f));
	}
	pObj = g_world.getDynamicObject("nemo2");
	pPath = g_world.findPath("fish2");
	if (pObj && pPath)
	{
		pObj->setPosition(pPath->getPointAt(time / 30.0f));
		pObj->setDirection(pPath->getDirectionAt(time / 30.0f));
	}
	pObj = g_world.getDynamicObject("nemo3");
	pPath = g_world.findPath("fish3");
	if (pObj && pPath)
	{
		pObj->setPosition(pPath->getPointAt(time / 35.0f));
		pObj->setDirection(pPath->getDirectionAt(time / 35.0f));
	}	
	pObj = g_world.getDynamicObject("koraalvlinder1");
	pPath = g_world.findPath("fish4");
	if (pObj && pPath)
	{
		pObj->setPosition(pPath->getPointAt(time / 27.0f));
		pObj->setDirection(pPath->getDirectionAt(time / 27.0f));
	}
	pObj = g_world.getDynamicObject("koraalvlinder2");
	pPath = g_world.findPath("fish5");
	if (pObj && pPath)
	{
		pObj->setPosition(pPath->getPointAt(time / 28.0f));
		pObj->setDirection(pPath->getDirectionAt(time / 28.0f));
	}
}

void animateBird(float time)
{
	float rot = sin(time*0.75f)*cos(time/3.0)*cos(time*1.25/3.0);
	rot *= 50.0;

	g_world.getDynamicObject("BirdHead")->setYaw(rot);
	g_world.getDynamicObject("BirdRightEye")->setYaw(rot);
	g_world.getDynamicObject("BirdLeftEye")->setYaw(rot);
	g_world.getDynamicObject("BirdNose")->setYaw(rot);
	g_world.getDynamicObject("BirdTail")->setPitch(sin(time) * 5.0f);
	g_world.getDynamicObject("BirdLeftWing")->setRoll ((sin(time * 2) + 1.0) *  5.0f);
	g_world.getDynamicObject("BirdRightWing")->setRoll((sin(time * 2) + 1.0) * -5.0f);
}

void idle(void)
{
	// Animate everything
	float time = g_world.getTimer().getTime();
	
	animateDinosaur(time);
	animateAquarium(time);
	animateBird(time);

	glutPostRedisplay();
}

// The mouse callback
void mouse(int button, int state, int x, int y)
{
	g_inputManager.onButton(button, state, x, y);
}

// The motion callback
void activeMotion(int x, int y)
{
	g_inputManager.onMotion(true, x, y);
}

// The passive motion callback
void passiveMotion(int x, int y)
{
	g_inputManager.onMotion(false, x, y);
}

void special(int key, int x, int y)
{
	g_inputManager.onSpecialKey(key, x, y);
}

void specialup(int key, int x, int y)
{
	g_inputManager.onSpecialKeyUp(key, x, y);
}

void keyboard(unsigned char key, int x, int y)
{
	g_inputManager.onKey(key, x, y);
}


// Window reshape callback
void reshape(GLsizei w, GLsizei h)
{
	glViewport(0, 0, w, h);
}

bool initWorld()
{
	cout << "Loading (this can take a while)..." << endl;
	if (!g_world.load(WORLD_FILE)) 
	{
		cerr << "Failed to load .cgworld file. Press a key to quit." << endl;
		system("pause>nul");
		return false;
	}

	g_world.init();
	g_pDino = NULL;
	try
	{
		g_pDino = new Dino();
	}
	catch(Model::Error e)
	{	cerr << "Failed loading dino model.Error:" << endl << e.what() << endl << "Press a key to quit." << endl;
		system("pause>nul");
		return false;
	}

	g_pDinoPath = g_world.findPath("dino");
	g_world.getCamera()->setPosition(CAMERA_POS);
	g_world.getCamera()->pitch(CAMERA_PITCH);
	g_world.getCamera()->yaw(CAMERA_YAW);

	g_world.getTimer().start();
	return true;
}


int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_RGB|GLUT_DOUBLE);
	glutInitWindowSize (800,600);
	glutInitWindowPosition(0,0);
    glutCreateWindow (WINDOW_TITLE);

	if (!initWorld())
		return 1;

	glutReshapeFunc(reshape);
	glutMouseFunc(mouse);
	glutMotionFunc(activeMotion);
	glutPassiveMotionFunc(passiveMotion);
	glutIgnoreKeyRepeat(1);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(special);
	glutSpecialUpFunc (specialup);
	glutIdleFunc(idle);
	glutDisplayFunc(display);

	glutMainLoop();
	return 0;
}

